/*
 * Copyright (C) 2009 by Eric Herman <eric@freesa.org>
 * Copyright (C) 2009 by Robert Stewart <robert@wombatnation.com>
 * Copyright (C) 2009 by Eric Lambert <Eric.Lambert@sun.com>
 * Use and distribution licensed under the
 * GNU Lesser General Public License (LGPL) version 2.1.
 * See the COPYING file in the parent directory for full text.
 */
package org.gearman.common;

import java.io.IOException;

/*
 * This is essentially a copy of the the gearmanij.Connection interface.
 */

/**
 * This interface exposes the API that classes representing connections to a
 * Gearman Job Server must implement.
 */
public interface GearmanJobServerConnection {

    /**
     * Opens a connection to the Gearman Job Server
     */
    void open() throws IOException;

    /**
     * Closes a connection to the Gearman Job Server
     */
    void close();

    /**
     * Writes a {@link GearmanPacket} into the connection with the Gearman
     * Job Server.
     *
     * @param request The request to be written.
     */
    void write(GearmanPacket request) throws IOException;

    /**
     * Reads a {@link GearmanPacket} from the connection with the Gearman
     * Job Server.
     * @return the GearmanPacket read from the connection.
     * @throws IORuntimeException if an I/O exception was encountered.
     */
    GearmanPacket read() throws IOException;

    /**
     * Is the connection open.
     *
     * @return true if the connection is open, else false.
     */
    boolean isOpen();

    boolean canRead();

    boolean canWrite();
}
