/*
 * Copyright (C) 2009 by Eric Lambert <Eric.Lambert@sun.com>
 * Use and distribution licensed under the
 * GNU Lesser General Public License (LGPL) version 2.1.
 * See the COPYING file in the parent directory for full text.
 */
package org.gearman.common;

/**
 * A <tt>GearmanJobEvent</tt> represents an event that occurs in either
 * the submission of job or the execution of a job.
 *
 */
public interface GearmanPacket {

    public static enum DataComponentName {
        FUNCTION_NAME,
        UNIQUE_ID,
        MINUTE,
        HOUR,
        DAY_OF_MONTH,
        MONTH,
        DAY_OF_WEEK,
        EPOCH,
        JOB_HANDLE,
        OPTION,
        KNOWN_STATUS,
        RUNNING_STATUS,
        NUMERATOR,
        DENOMINATOR,
        TIME_OUT,
        DATA,
        ERROR_CODE,
        ERROR_TEXT,
        CLIENT_ID
    }

    /**
     * Retrieves the event type for this event.
     *
     * @return event type.
     */
    public PacketType getPacketType();

    public byte [] toBytes();

    /**
     * Retrieves the payload, if any, associated with this event.
     *
     * @return payload
     */
    public byte [] getData();
    
    public byte [] getDataComponentValue( DataComponentName component);

    public PacketMagic getMagic();

    public boolean requiresResponse();

}
