/*
 * Copyright (C) 2009 by Eric Herman <eric@freesa.org>
 * Use and distribution licensed under the 
 * GNU Lesser General Public License (LGPL) version 2.1.
 * See the COPYING file in the parent directory for full text.
 */

package org.gearman.example;

import org.gearman.common.Constants;
import org.gearman.common.GearmanPacket;
import org.gearman.common.Packet;
import org.gearman.common.PacketMagic;
import org.gearman.common.PacketType;
import org.gearman.common.SocketConnection;
import org.gearman.util.ByteUtils;
import java.io.PrintStream;


public class EchoClient {

  private SocketConnection connection;

  private boolean loop = true;

  public EchoClient(String host, int port) {
    this.connection = new SocketConnection(host, port);
    connection.open();
  }

  public String echo(String input) {
    byte[] data = ByteUtils.toAsciiBytes(input);
    GearmanPacket reverseRequest = new Packet(PacketMagic.REQ,PacketType.ECHO_REQ,data);
    connection.write(reverseRequest);
    byte[] respBytes = readResponse();
    String echo = ByteUtils.fromAsciiBytes(respBytes);
    return echo;
  }

  private byte[] readResponse() {
    byte[] respBytes = ByteUtils.EMPTY;
    while (loop) {
      Packet fromServer = connection.read();
      PacketType packetType = fromServer.getPacketType();
      if (packetType == PacketType.ECHO_RES) {
        return fromServer.getData();
      } else {
        println("Unexpected PacketType: " + packetType);
        println("Unexpected Packet: " + fromServer);
      }
    }
    return respBytes;
  }

  public void shutdown() {
    loop = false;
  }

  public static void main(String[] args) {
    if (args.length == 0 || args.length > 3) {
      usage(System.out);
      return;
    }
    String host = Constants.GEARMAN_DEFAULT_TCP_HOST;
    int port = Constants.GEARMAN_DEFAULT_TCP_PORT;
    String payload = args[args.length - 1];
    for (String arg : args) {
      if (arg.startsWith("-h")) {
        host = arg.substring(2);
      } else if (arg.startsWith("-p")) {
        port = Integer.parseInt(arg.substring(2));
      }
    }

    System.out.println(new EchoClient(host, port).echo(payload));
  }

  public static void usage(PrintStream out) {
    String[] usage = {
        "usage: gearmanij.example.EchoClient [-h<host>] [-p<port>] <string>",
        "\t-h<host> - job server host",
        "\t-p<port> - job server port",
        "\n\tExample: java gearmanij.example.EchoClient Foo",
        "\tExample: java gearmanij.example.EchoClient -h127.0.0.1 -p4730 Bar", //
    };

    for (String line : usage) {
      out.println(line);
    }
  }

  public static void println(String msg) {
    System.err.println(Thread.currentThread().getName() + ": " + msg);
  }

}

