/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.gearman.worker;


import org.gearman.common.PacketType;
import java.util.HashSet;
import java.util.Set;
import org.gearman.worker.GearmanFunction;
import org.gearman.common.GearmanPacket;
import org.gearman.client.GearmanIOEventListener;
import org.gearman.client.GearmanJobResult;
import org.gearman.common.Packet;
import org.gearman.common.PacketMagic;
import org.gearman.util.ByteUtils;


/**
 *
 * @author elambert
 */
public abstract class AbstractGearmanFunction implements GearmanFunction {

    protected String name;
    protected Object data;
    protected byte[] jobHandle;
    protected Set<GearmanIOEventListener> listeners;

    public AbstractGearmanFunction() {
        listeners = new HashSet<GearmanIOEventListener>();
        jobHandle = new byte[0];
        name = this.getClass().getCanonicalName();
    }

    public String getName() {
        return name;
    }

    public void setData(Object data) {
        this.data = data;
    }

    public void setJobHandle(byte[] handle) throws IllegalArgumentException {
        if (handle == null) {
            throw new IllegalArgumentException("handle can not be null");
        }
        if (handle.length == 0) {
            throw new IllegalArgumentException("handle can not be empty");
        }
        jobHandle = new byte[handle.length];
        System.arraycopy(handle, 0, jobHandle, 0, handle.length);
    }

    public byte[] getJobHandle() {
        byte[] rt = new byte[jobHandle.length];
        System.arraycopy(jobHandle, 0, rt, 0, jobHandle.length);
        return rt;
    }

    public void registerEventListener(GearmanIOEventListener listener)
            throws IllegalArgumentException {
        if (listener == null) {
            throw new IllegalArgumentException("listener can not be null");
        }
        listeners.add(listener);
    }

    public void fireEvent(GearmanPacket event)
            throws IllegalArgumentException {
        if (event == null) {
            throw new IllegalArgumentException("event can not be null");
        }
        for (GearmanIOEventListener listener : listeners) {
            listener.handleGearmanIOEvent(event);
        }
    }

    public void sendData(byte [] data) {
        fireEvent(new Packet(PacketMagic.REQ, PacketType.WORK_DATA,
                Packet.generatePacketData(jobHandle,data)));

    }

    public void sendWarning(byte [] warning) {
        fireEvent(new Packet(PacketMagic.REQ, PacketType.WORK_WARNING,
                Packet.generatePacketData(jobHandle,warning)));
    }
    
    public void sendException(byte [] exception) {
        fireEvent(new Packet(PacketMagic.REQ, PacketType.WORK_EXCEPTION,
                Packet.generatePacketData(jobHandle,exception)));
    }
    
    public void sendStatus(int denominator, int numerator) {
        fireEvent(new Packet(PacketMagic.REQ, PacketType.WORK_STATUS,
                Packet.generatePacketData(jobHandle,
                ByteUtils.toUTF8Bytes(String.valueOf(numerator)),
                ByteUtils.toUTF8Bytes(String.valueOf(denominator))))); 
    }
    
    public abstract GearmanJobResult executeFunction() throws Exception;

    public GearmanPacket call() throws Exception {
        GearmanPacket event = null;
        try {
            GearmanJobResult result = executeFunction();
            event = new Packet(PacketMagic.REQ, PacketType.WORK_COMPLETE,
                    Packet.generatePacketData(jobHandle,result.getResults()));
        } catch (Exception e) {
            fireEvent(new Packet(PacketMagic.REQ, PacketType.WORK_EXCEPTION,
                    Packet.generatePacketData(jobHandle,e.getMessage().getBytes())));
            event = new Packet(PacketMagic.REQ, PacketType.WORK_FAIL, jobHandle);
        }
        fireEvent(event);
        return event;
    }
}
