/*
 * Copyright (C) 2009 by Eric Lambert <Eric.Lambert@sun.com>
 * Use and distribution licensed under the
 * GNU Lesser General Public License (LGPL) version 2.1.
 * See the COPYING file in the parent directory for full text.
 */
package org.gearman.worker;

import java.util.concurrent.Callable;
import org.gearman.client.GearmanIOEventListener;
import org.gearman.common.GearmanPacket;

/**
 *
 * This interface exposes the API which GearmanFunction classes will need to
 * implement.
 *
 * <p>
 * A Gearman Function represents the work to be accomplished by a
 * {@link GearmanJob}. It defines how the job is executed and the data, if
 * any, against which that execution will occurr.
 */
public interface GearmanFunction extends Callable<GearmanPacket> {

    /**
     * Retrieves the name of the funcion. This is used by a {@link GearmanWorker}
     * to determine if the function has been regisitered with the user and
     * therefor can be executed by the Worker.
     *
     * @return the name of the function.
     */
    String getName();

    /**
     * Some functions require a dataset upon which to operate. This method is
     * used to load that data so that it is available when the function is
     * called.
     *
     * @param data The data used by the function at execution time.
     */
    void setData(Object data);

    /**
     * Set the handle of the job for which this function is executing.
     *
     * @param handle the job handle.
     * @throws IllegalArgumentException if the handle is null or empty.
     */
    void setJobHandle(byte [] handle) throws IllegalArgumentException;

    byte [] getJobHandle();

    void registerEventListener(GearmanIOEventListener listener);

    void fireEvent(GearmanPacket event);


}
