/*
 * Copyright (C) 2009 by Trond Norbye <trond.norbye@sun.com>
 * Use and distribution licensed under the
 * GNU Lesser General Public License (LGPL) version 2.1.
 * See the COPYING file in the parent directory for full text.
 */
package org.gearman.worker;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

/**
 * The ObjectFunction class lets you create a worker that receives Objects
 * from the client and return a Serializable object back to the client.
 * Please note that the Java class needs to be availabel on both the server
 * and the client ;-)
 *
 * @author Trond Norbye
 */
public abstract class ObjectFunction implements JobFunction {

    final public byte[] execute(byte[] data) {
        byte[] ret = null;
        try {
            ObjectInputStream in = new ObjectInputStream(new ByteArrayInputStream(data));
            Serializable result = execute(in.readObject());
            if (result != null) {
                ByteArrayOutputStream out = new ByteArrayOutputStream();
                ObjectOutputStream ostream = new ObjectOutputStream(out);
                ostream.writeObject(result);
                ostream.close();
                ret = out.toByteArray();
            }
        } catch (Exception ex) {
            throw new RuntimeException("Failed to execute job", ex);
        }

        return ret;
    }

    /**
     * The method the user should override
     *
     * @param object The object passed by the client
     * @return the result to return to the client
     */
    public abstract Serializable execute(Object object);
}
