/*
 * Copyright (C) 2009 by Eric Lambert <Eric.Lambert@sun.com>
 * Use and distribution licensed under the
 * GNU Lesser General Public License (LGPL) version 2.1.
 * See the COPYING file in the parent directory for full text.
 */
package org.gearman.common;


import java.io.IOException;
import java.net.ServerSocket;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;

import junit.framework.Assert;

import org.gearman.client.GearmanEchoResponseHandler;

import org.junit.Test;

//TODO refactor all initialization code that most tests perform into
public class GearmanJobServerSessionTest {

    @Test
    public void ctorTest() {
        GearmanNIOJobServerConnection localhost = new
                GearmanNIOJobServerConnection("localhost");
        GearmanJobServerSession session = new GearmanJobServerSession(localhost);
        Assert.assertFalse("session incorrectly claims to be initialized",
                session.isInitialized());
        Assert.assertEquals(GearmanJobServerSession.DESCRIPTION_PREFIX + ":" +
                Thread.currentThread().getId() + ":" +
                localhost, session.toString());
        Throwable thrown = null;
        try {
            session = new GearmanJobServerSession(null);
            Assert.fail("No exception was thrown on attempt to create a new" +
                    " sesssion with a null remote address");
        } catch (Throwable t) {
            thrown = t;
        }
        Assert.assertTrue("Unexpected exception was thrown on attempt to" +
                " create a new sesssion with a null remote address",
                thrown instanceof IllegalArgumentException);
    }

    @Test
    public void getConnectionTest() {
        GearmanNIOJobServerConnection conn =
                new GearmanNIOJobServerConnection("localhost");
        GearmanJobServerSession session = new GearmanJobServerSession(conn);
        Assert.assertEquals("session getConnection return value does not equal" +
                " server used to create session",conn, session.getConnection());
    }

//    @Test
//    public void initializeSessionTest() throws IOException {
//        GearmanNIOJobServerConnection conn =
//                new GearmanNIOJobServerConnection("localhost",
//                Constants.GEARMAN_DEFAULT_TCP_PORT - 1);
//        GearmanJobServerSession session = null;;
//        //try and initialize using a remote that we can not connect to
//        session = new GearmanJobServerSession(conn);
//        try {
//            session.initSession(Selector.open(),0);
//            Assert.fail("initSession call did not throw exception even " +
//                    "though session can not connect to host");
//        } catch (Throwable t) {
//            Assert.assertTrue("unexpected exception thrown when initializing" +
//                    " session with a non-connectable host. Expected  " +
//                    "IOException, got " + t, t instanceof IOException);
//        }
//
//        ServerSocket s = new ServerSocket(Constants.GEARMAN_DEFAULT_TCP_PORT - 1);
//        session.initSession(Selector.open(),0);
//        try {
//            session.initSession(Selector.open(),0);
//            Assert.fail("initSession was called twice but did throw " +
//                    "exception on second call");
//        } catch (Throwable t) {
//            Assert.assertTrue("unexpected exception thrown when making " +
//                    "duplicate initialize session call. " +
//                    "Expected IllegalStateException, got " + t,
//                    t instanceof IllegalStateException);
//        }
//        session.closeSession();
//        s.close();
//    }

//    @Test
//    public void closeSessionTest() throws IOException {
//        GearmanNIOJobServerConnection conn =
//                new GearmanNIOJobServerConnection("localhost",
//                Constants.GEARMAN_DEFAULT_TCP_PORT - 1);
//        GearmanJobServerSession session = null;
//        session = new GearmanJobServerSession(conn);
//        ServerSocket s = new ServerSocket(
//                Constants.GEARMAN_DEFAULT_TCP_PORT - 1);
//        try {
//            session.closeSession();
//            Assert.fail("Attempt to close non-initialized session did not " +
//                    "result an exception being raised");
//        } catch (Throwable t) {
//            Assert.assertTrue("unexpected exception thrown when attempting" +
//                    " to close non-initialized session . " +
//                    "Expected IllegalStateException, got " + t,
//                    t instanceof IllegalStateException);
//        }
//
//        session.initSession(Selector.open(),0);
//        session.closeSession();
//        try {
//            session.closeSession();
//            Assert.fail("Duplicate close session did not result an" +
//                    " exception being raised");
//        } catch (Throwable t) {
//            Assert.assertTrue("unexpected exception thrown when making" +
//                    " duplicate initialize session call. " +
//                    "Expected IllegalStateExceptio, got " + t,
//                    t instanceof IllegalStateException);
//
//        }
//        s.close();
//    }

//    @Test
//    public void getKeyForSelectorTest() throws IOException {
//        GearmanNIOJobServerConnection conn =
//                new GearmanNIOJobServerConnection("localhost",
//                Constants.GEARMAN_DEFAULT_TCP_PORT - 1);
//        GearmanJobServerSession session = null;
//        session = new GearmanJobServerSession(conn);
//        try {
//            session.getSelectionKey();
//            Assert.fail("Attempt to obtain selectKeys for non-initialized" +
//                    " session did not result an exception being raised");
//        } catch (Throwable t) {
//            Assert.assertTrue("unexpected exception thrown when attempting" +
//                    " to obtain selectKeys for non-initialized session. " +
//                    "Expected IllegalStateException, got " + t,
//                    t instanceof IllegalStateException);
//        }
//
//        ServerSocket s = new ServerSocket(
//                Constants.GEARMAN_DEFAULT_TCP_PORT - 1);
//        session.initSession(Selector.open(),0);
//        Assert.assertTrue(session.getSelectionKey() != null);
//        try {
//            SelectionKey key = session.getSelectionKey();
//            Assert.assertTrue(key != null);
//        } finally {
//            session.closeSession();
//            s.close();
//        }
//    }

//    @Test
//    public void submitRequestToSessionTest() throws IOException {
//        GearmanNIOJobServerConnection conn =
//                new GearmanNIOJobServerConnection("localhost",
//                Constants.GEARMAN_DEFAULT_TCP_PORT - 1);
//        GearmanJobServerSession session = null;
//        session = new GearmanJobServerSession(conn);
//        GearmanPacket echoRequest = new Packet(PacketMagic.REQ,
//                PacketType.ECHO_REQ,new byte[0]);
//        GearmanTask gsr = new GearmanTask(
//                new GearmanEchoResponseHandler(), echoRequest);
//        ServerSocket s = null;
//
//        try {
//            session.submitTask(gsr);
//            Assert.fail("Attempt to submit request to non-initialized " +
//                    "session did not result an exception being raised");
//        } catch (Throwable t) {
//            Assert.assertTrue("unexpected exception thrown when attempting to" +
//                    " submit request to non-initialized session. " +
//                    "Expected IllegalStateException, got " + t,
//                    t instanceof IllegalStateException);
//        }
//
//        try {
//            s = new ServerSocket(Constants.GEARMAN_DEFAULT_TCP_PORT - 1);
//            session.initSession(Selector.open(),0);
//
//            try {
//                session.submitTask(null);
//                Assert.fail("Attempt to submit null request to session did" +
//                        " not result an exception being raised");
//            } catch (Throwable t) {
//                Assert.assertTrue("unexpected exception thrown when" +
//                        " attempting to submit null request session. " +
//                        "Expected IllegalStateException, got " + t,
//                        t instanceof IllegalStateException);
//            }
//
//            try {
//                session.submitTask(gsr);
//            } catch (Throwable t) {
//                Assert.fail("unexpected exception thrown when attempting" +
//                        " to submit valied request session: " + t);
//            }
//        } finally {
//            try {
//                session.closeSession();
//                s.close();
//            } catch (Exception swallow) {
//            }
//        }
//    }
//
//    @Test
//    public void isProcessingRequestTest() throws IOException {
//        GearmanNIOJobServerConnection conn =
//                new GearmanNIOJobServerConnection("localhost",
//                Constants.GEARMAN_DEFAULT_TCP_PORT - 1);
//        GearmanJobServerSession session = null;
//        session = new GearmanJobServerSession(conn);
//        GearmanPacket echoRequest = new Packet(PacketMagic.REQ,
//                PacketType.ECHO_REQ, new byte[0]);
//        GearmanTask gsr = new GearmanTask(
//                new GearmanEchoResponseHandler(), echoRequest);
//        ServerSocket s = null;
//
//        Assert.assertFalse("Calling isProcessingRequest on un-initialized" +
//                " session returned true", session.isProcessingRequests());
//        try {
//            s = new ServerSocket(Constants.GEARMAN_DEFAULT_TCP_PORT - 1);
//            session.initSession(Selector.open(),0);
//            session.submitTask(gsr);
//            Assert.assertTrue("Calling isProcessingRequest on session that" +
//                    " is processing request returned false",
//                    session.isProcessingRequests());
//            //TODO implement a means to actual process the request and verify that isProcessingResults returns false
//        } finally {
//            try {
//                session.closeSession();
//                s.close();
//            } catch (Exception swallow) {
//            }
//        }
//    }
//
//    @Test
//    public void driveSessinIOTest() throws IOException {
//        GearmanNIOJobServerConnection conn =
//                new GearmanNIOJobServerConnection("localhost",
//                Constants.GEARMAN_DEFAULT_TCP_PORT - 1);
//        GearmanJobServerSession session = null;
//        session = new GearmanJobServerSession(conn);
//        GearmanPacket echoRequest = new Packet(PacketMagic.REQ,
//                PacketType.ECHO_REQ, new byte[0]);
//        GearmanTask gsr = new GearmanTask(
//                new GearmanEchoResponseHandler(), echoRequest);
//        ServerSocket s = null;
//
//        try {
//            session.driveSessionIO();
//            Assert.fail("Attempt to driveIO on a non-initialized session " +
//                    "did not result an exception being raised");
//        } catch (Throwable t) {
//            Assert.assertTrue("unexpected exception thrown when attempting" +
//                    " to drive IO on a non-initialized session. " +
//                    "Expected IllegalStateException, got " + t,
//                    t instanceof IllegalStateException);
//        }
//
//        try {
//            s = new ServerSocket(Constants.GEARMAN_DEFAULT_TCP_PORT - 1);
//            session.initSession(Selector.open(),0);
//            session.submitTask(gsr);
//            session.driveSessionIO();
//            //make sure that we are still processing (since driveIO should do nothing of value since we have no real connection
//            Assert.assertTrue("Calling isProcessingRequest on session that" +
//                    " should be processing request returned false",
//                    session.isProcessingRequests());
//        } finally {
//            try {
//                session.closeSession();
//                s.close();
//            } catch (Exception swallow) {
//            }
//        }
//    }
}
