/*
 * Copyright (C) 2009 by Eric Lambert <Eric.Lambert@sun.com>
 * Use and distribution licensed under the
 * GNU Lesser General Public License (LGPL) version 2.1.
 * See the COPYING file in the parent directory for full text.
 */


package org.gearman.worker;

import org.gearman.common.Constants;
import org.gearman.common.PacketMagic;
import org.gearman.common.PacketType;
import org.gearman.common.Packet;
import java.io.IOException;
import java.net.ServerSocket;
import org.gearman.common.GearmanNIOJobServerConnection;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class GearmanWorkerJobServerSessionTest {

    GearmanWorkerJobServerSession session;
    ServerSocket s;

    @Before
    public void initTest() throws IOException {
        s = new ServerSocket(Constants.GEARMAN_DEFAULT_TCP_PORT - 1);
        session = new GearmanWorkerJobServerSession(
                new GearmanNIOJobServerConnection("localhost",
                Constants.GEARMAN_DEFAULT_TCP_PORT - 1));
    }

    @After
    public void shutDownTest() {
        try {
            s.close();
        } catch (IOException ioe) {}
    }

    @Test
    public void ctorTest() {
        Assert.assertTrue(session.isAwake());
        Assert.assertFalse(session.isInitialized());
        Assert.assertTrue(session.getNewJobs().isEmpty());
    }

    @Test
    public void sleepWakeTest() {
        session.setToSleep();
        Assert.assertFalse(session.isAwake());
        session.wakeSession();
        Assert.assertTrue(session.isAwake());
    }

    @Test
    public void submitBadJobTest() {
        Assert.assertFalse(submitBadJob(null));
        Assert.assertEquals(0, session.getNewJobs().size());
        Assert.assertFalse(submitBadJob(new Packet(PacketMagic.REQ,
                PacketType.NOOP, new byte[0])));
        Assert.assertEquals(0, session.getNewJobs().size());
    }

    private boolean submitBadJob(Packet p) {
        try {
            session.addNewJob(p);
        } catch (IllegalArgumentException iae) {
            return false;
        }
        return true;
    }
}
